/*
 *
 * Tachometer with DL430 display ver 1.0
 *
 * Juha Niinikoski 11.03.2005
 * 
 * Average filter added 13.03.2005
*/

#include <pic.h>
#include "revctr.h"

/* global variables */

near bit feed_dog;				// watch dog feed flag
near bit s_flag;				// second elapsed
near unsigned char s0;				// second wait counters

static near unsigned char digit_data[3];	// segment bits
static near unsigned char dp_data[3];		// DP bits

bit run_flag;					// signal ok flag
bit mode;						// operating mode

bit meas_ready;					// measurement ready
unsigned int t1_result;			// timer1 result
unsigned int rpm;				// rpm / 10

#define NUM_AVG 4				// average size
unsigned int avg[NUM_AVG];		// average array
char idx_avg;					// array index
unsigned int rpm_avg;			// averaged rpm

// nominal values
#define two_stroke   750000UL	// constant for two stroke or waste spark 4 stroke engine, spark at every round
#define four_stroke 1500000UL	// constant for four stroke engine, two cranc rounds / spark

// calibrated values for particular controller RC clock 


/* included programs */

#include "7segmux.c"

/* processor config & id info */

	__CONFIG(UNPROTECT & INTIO & WDTDIS & PWRTEN & BOREN & LVPDIS & MCLRDIS);	// 16F628 & 16F81x

	__IDLOC(0103);							// Device ID

const char id[] = {"Juha Niinikoski 13.03.2005"};	// Visible id string


/* helper routines */


/* watchdog feed routine */


void do_dog( void )
	{
	if(feed_dog)				// wait feed signal
		{						// from IRQ routine
		feed_dog = 0;
		asm("clrwdt");			// make watch dog happy
		}
	}

/* main  */

void main(void)
{
char x;

	CMCON = CMCON_INIT;			// comparators off 16F628 only
	VRCON = VRCON_INIT;			// voltage reference init

	PORTB = PORTB_INIT;
	PORTA = PORTA_INIT;

	TRISB = TRISB_INIT;			// RB5 - RB7 otputs
	TRISA = TRISA_INIT;

	init_timers();				// start timing functions

//	putst("---");				// start up display
	digit_data[2] = C_NEG;		// display start up message
	digit_data[1] = C_1;		// start up display / version number
	digit_data[0] = C_NEG;

	delayms(2000);

	for(;;)
		{
		do_dog();						// watchdog care if enabled

// if measurement ready
		if(meas_ready)
			{
			meas_ready = 0;				// reset ready flag
			msw = 1000;					// reset timeout

			run_flag = !run_flag;		// toggle display run flag

			if(mode)					// print result, spark / two rounds
				{
				rpm = four_stroke / t1_result;
				}
			else						// spark / round
				{
				rpm = two_stroke / t1_result;
				}

// average filter for result
			avg[idx_avg] = rpm;			// store last result
			idx_avg++;					// addjust index
			if(idx_avg >= NUM_AVG)
				idx_avg = 0;

			rpm_avg = 0;				// calculate average
			for(x=0; x< NUM_AVG; x++)
				{
				rpm_avg = rpm_avg + avg[x];
				}

			rpm_avg = rpm_avg + (NUM_AVG / 2);	// round
			rpm_avg = rpm_avg / NUM_AVG;

// display average
			putintdec(rpm_avg);

// no averaging
//			putintdec(rpm);

			}

// slow signal trap
		if(msw == 0)
				{
				putst("?-?");			// no signal display, save power
				run_flag = 0;			// blanc point, save power
				}
// set DP point flags for display
		if (run_flag == 1)
			dp_data[2] = Dp;
		else
			dp_data[2] = 0;

		if (mode == 1)
			dp_data[0] = Dp;			// mode point (4 stroke select) after last digit
		else
			dp_data[0] = 0;
		}
}
